﻿import util.*;
import mvc.*;

/**
 * 提供基本服務給MVC三劍客中的外觀
 */
class mvc.AbstractView implements Observer, View {
  private var model:Observable;       // 模型的參考
  private var controller:Controller;  // 控制邏輯的參考

  public function AbstractView (m:Observable, c:Controller) {
    // 設定模型
    setModel(m);

    // 如果程式有提供控制邏輯，就採用它。否則就呼叫
    // getController( ) 來建立預設的控制邏輯。
    if (c !== undefined) {
      setController(c);
    }
  }

  /**
   * 替這個外觀傳回預設的控制邏輯
   */
  public function defaultController (model:Observable):Controller {
    return null;
  }

  /**
   * 設定這個View所觀察的Model
   */
  public function setModel (m:Observable):Void {
    model = m;
  }

  /**
   * 傳回這個外觀所觀察的模型
   */
  public function getModel (  ):Observable {
    return model;
  }

  /**
   * 替這個外觀設定控制邏輯
   */
  public function setController (c:Controller):Void {
    controller = c;
    // 告訴控制邏輯這個物件就是它的外觀
    getController(  ).setView(this);
  }

  /**
   * 傳回這個外觀的控制邏輯
   */
  public function getController (  ):Controller {
    // 如果這個控制邏輯尚未被定義…
    if (controller === undefined) {
      // ...就產生一個。請留意，defaultController(  ) 通常被AbstractView
      // 的子類別覆寫，以便讓它傳回適當的控制邏輯給這個外觀。
setController(defaultController(getModel(  )));
    }
    return controller;
  }

  /**
   * 在此Observer介面中定義update()方法，但不需實作。
   * AbstractView的子類別將為這個方法提供有用的實作。
   */
  public function update(o:Observable, infoObj:Object):Void {
  }
}