﻿import util.*;
import mvcclock.*;
import mvc.*;

/**
 * 用於ClockModel類別的類比時鐘外觀。這個外觀
 * 不接受用戶的輸入，因此它不需要控制邏輯。
 */
class mvcclock.ClockAnalogView extends AbstractView {
  // 包含ClockAnalogViewSymbol的一個實體，
  // 它被用來在螢幕上描繪時鐘的外觀。
  private var clock_mc:MovieClip;

  /**
   * 建構子
   */
  public function ClockAnalogView (m:Observable, c:Controller,
                                   target:MovieClip, depth:Number,
                                   x:Number, y:Number) {
    // 呼叫父建構子，它將設定MVC的關係。
    // 這個外觀不接受用戶輸入，因此不需要控制邏輯。
    super(m, c);

    // 建立使用者介面
    makeClock(target, depth, x, y);
  }

  /**
   * 建立將以類比格式呈現時間的影片片段實體
   *
   * @param   target 	要建立影片片段的片段
   * @param   depth 	被建立的影片片段所在的深度
   * @param   x   		影片片段在target裡的水平座標
   * @param   y  	 	影片片段在target裡的垂直座標
   */
  public function makeClock (target:MovieClip, depth:Number, 
                             x:Number, y:Number):Void {
    clock_mc = target.attachMovie("ClockAnalogViewSymbol",
               "analogClock_mc", depth);
    clock_mc._x = x;
    clock_mc._y = y;
  }

  /**
   * 更新類比時鐘在螢幕上的狀態。
   * 被ClockModel.notifyObservers(  ) 自動呼叫。
   *
   * @param   o   		廣播更新訊息的ClockModel物件
   * @param   infoObj   描述發生在ClockModel中的改變的
   *                    ClockUpdate實體
   */
  public function update (o:Observable, infoObj:Object):Void {
    // 把通用的infoObj轉換成ClockUpdate資料類型
    var info:ClockUpdate = ClockUpdate(infoObj);

    // 顯示新的時間
    var dayPercent:Number = (info.hour>12 ? info.hour-12 : info.hour) / 12;
    var hourPercent:Number = info.minute/60;
    var minutePercent:Number = info.second/60;
    clock_mc.hourHand_mc._rotation = 360 * dayPercent 
                                     + hourPercent * (360 / 12);
    clock_mc.minuteHand_mc._rotation = 360 * hourPercent;
    clock_mc.secondHand_mc._rotation = 360 * minutePercent;

    // 如果時鐘沒有在執行的話，就淡化顯示時鐘。
    if (info.isRunning) {
      clock_mc._alpha = 100;
    } else {
      clock_mc._alpha = 50;
    }
  }
}