﻿import util.*;
import mvcclock.*;
import mvc.*;

/**
 * 用於ClockModel的數位外觀。這個外觀不接受用戶輸入，
 * 所以不需要控制邏輯。
 */
class mvcclock.ClockDigitalView extends AbstractView {
  // 小時的格式
  private var hourFormat:Number = 24;
  // 時鐘顯示畫面裡的分隔字元
  private var separator:String = ":";
  // 顯示時間的文字欄位，由makeClock(  )建立。
  private var clock_txt:TextField;   

  /**
   * 建構子
   */
  public function ClockDigitalView (m:Observable,  c:Controller, 
                                    hf:Number, sep:String, target:MovieClip,
                                    depth:Number, x:Number, y:Number) {
    // 呼叫設置MVC關係的父建構子
    super(m, c);

    // 確認指定的小時格式是合法的，如果是，就使用它。
    if (hf == 12) {
      hourFormat = 12;
    }

    // 如果有提供分隔字元，就使用它。
    if (sep != undefined) {
      separator = sep;
    }

    // 建立使用者介面
    makeClock(target, depth, x, y);
  }

  /**
   * 建立將用在螢幕上顯示數字格式的時間的文字欄位
   *
   * @param   target   要在其中建立文字欄位的影片片段
   * @param   depth   被建立的文字欄位所在的深度
   * @param   x   文字欄位在target裡的水平座標
   * @param   y   文字欄位在target裡的垂直座標
   */
  public function makeClock (target:MovieClip, depth:Number,
                             x:Number, y:Number):Void {
    // 建立文字欄位
    target.createTextField("clock_txt", depth, x, y, 0, 0);
    // 儲存此文字欄位的參考
    clock_txt = target.clock_txt;
    // 設定文字欄位的特徵（屬性）
    clock_txt.autoSize = "left";
    clock_txt.border = true;    
    clock_txt.background = true;
  }

  /**
   * 更新螢幕上的數字時間狀態
   * 由ClockModel自動呼叫
   *
   * @param   o 		廣播更新訊息的ClockModel物件
   * @param   infoObj	描述發生在ClockModel中的改變的
   *					ClockUpdate實體
   */
  public function update (o:Observable, infoObj:Object):Void {
    // 把通用的infoObj物件轉換成ClockUpdate資料類型
    var info:ClockUpdate = ClockUpdate(infoObj);

    // 使用恰當的格式，建立代表時間的字串。
    var timeString:String = (hourFormat == 12)
           ? 
           formatTime12(info.hour, info.minute, info.second)
           :
           formatTime24(info.hour, info.minute, info.second);

    // 在文字欄位中顯示新的時間
    clock_txt.text = timeString;

    // 如果時鐘沒有在執行的話，就淡化顯示時鐘。
    if (info.isRunning) {
      clock_txt.textColor = 0x000000;
    } else {
      clock_txt.textColor = 0x666666;
    }
  }

  /**
   * 傳回24時制的時間字串
   *
   * @param   h   小時，從0到23。
   * @param   m   分鐘，從0到59。
   * @param   s   秒，從0到59。
   */
  private function formatTime24 (h:Number, m:Number, s:Number):String {
    var timeString:String = "";

    // 格式化小時…
    if (h < 10) {
      timeString += "0";
    }
    timeString += h + separator;

    // 格式化分鐘…
    if (m < 10) {
      timeString += "0";
    }
    timeString += m + separator;

    // 格式化秒…
    if (s < 10) {
      timeString += "0";
    }
    timeString += String(s);

    return timeString;
  }

  /**
   * 傳回12時制的時間字串（不含AM和PM標示）
   *
   * @param   h   小時，從0到23。
   * @param   m   分鐘，從0到59。
   * @param   s   秒，從0到59。
   */
  private function formatTime12 (h:Number, m:Number, s:Number):String {
    var timeString:String = "";

    // 格式化小時…
    if (h == 0) {
      timeString += "12" + separator;
    } else if (h > 12) {
      timeString += (h - 12) + separator;
    } else {
      timeString += h + separator;
    }

    // 格式化分鐘…
    if (m < 10) {
      timeString += "0";
    }
    timeString += m + separator;

    // 格式化秒…
    if (s < 10) {
      timeString += "0";
    }
    timeString += String(s);

    return timeString;
  }
}