﻿import util.Observable;
import mvcclock.*;

/**
 * 呈現時鐘的資料（亦即，MVC三劍客中的「模型」）。
 */
class mvcclock.ClockModel extends Observable {
  // 目前的小時
  private var hour:Number;
  // 目前的分鐘
  private var minute:Number;
  // 目前的秒
  private var second:Number;
  // interval識別字用來指定每秒呼叫
  // tick( )的頻率。
  private var tickInterval:Number;
  // 指出時鐘是否有在執行
  private var isRunning:Boolean;

  /**
   * 建構子
   */
  public function ClockModel (  ) {
    // 在預設情況下，把時鐘的時間設定成目前的系統時間。
    var now:Date = new Date(  );
    setTime(now.getHours(), now.getMinutes(), now.getSeconds(  ));
  }

  /**
   * 啟動時鐘
   */
  public function start (  ):Void {
    if (!isRunning) {
      isRunning = true;
      tickInterval = setInterval(this, "tick", 1000);

      var infoObj:ClockUpdate = new ClockUpdate(hour, minute,
                                                second, isRunning);
      setChanged(  );
      notifyObservers(infoObj);
    }
  }

  /**
   * 停止時鐘
   */
  public function stop (  ):Void {
    if (isRunning) {
      isRunning = false;
      clearInterval(tickInterval);

      var infoObj:ClockUpdate = new ClockUpdate(hour, minute, 
                                                second, isRunning);
      setChanged(  );
      notifyObservers(infoObj);
    }
  }

  /**
   * 設定目前的時間（亦即，hour、minute和second屬性）。
   * 並通知所有觀察者（「外觀」）時間的變化。
   *
   * @param   h   新的小時
   * @param   m   新的分鐘
   * @param   s   新的秒數
   */
  public function setTime (h:Number, m:Number, s:Number):Void {
    if (h != null && h != hour && isValidHour(h)) {
      hour = h;
      setChanged(  );
    }

    if (m != null && m != minute && isValidMinute(m)) {
      minute = m;
      setChanged(  );
    }

    if (s != null && s != second && isValidSecond(s)) {
      second = s;
      setChanged(  );
    }

    // 如果模型改變了，就通知外觀。
    if (hasChanged(  )) {
      var infoObj:ClockUpdate = new ClockUpdate(hour, minute, 
                                                second, isRunning);
      // 把更改後的時間傳給外觀
      notifyObservers(infoObj);
    }
  }

  /**
   * 檢查這個數字是否為有效的小時數字 
   * （亦即，介於0到23之間的整數）。
   *
   * @param   h   被檢驗的小時數字
   */
  private function isValidHour (h:Number):Boolean {
    return (Math.floor(h) == h && h >= 0 && h <= 23);
  }

  /**
   * 檢查這個數字是否為有效的分鐘數字 
   * （亦即，介於0到59之間的整數）
   *
   * @param   m	被檢驗的分鐘數字
   */
  private function isValidMinute (m:Number):Boolean {
    return (Math.floor(m) == m && m >= 0 && m <= 59);
  }

  /**
   * 檢查這個數字是否為有效的秒數
   * （亦即，介於0到59之間的整數）
   *
   * @param   s	被檢驗的秒數
   */
  private function isValidSecond (s:Number):Boolean {
    return (Math.floor(s) == s && s >= 0 && s <= 59);
  }

  /**
   * 替目前的時刻加上一秒鐘，讓時間流逝。
   */
  private function tick (  ):Void {
    // 取得現在的時刻
    var h:Number = hour;
    var m:Number = minute;
    var s:Number = second;

    // 增加目前的秒數，必要時，
    // 一併增加分鐘和時數。
    s += 1;
    if (s > 59) {
      s = 0;
      m += 1;
      if (m > 59) {
        m = 0;
        h += 1;
        if (h > 23) {
          h = 0;
        }
      }
    }

    // 設定新的時間
    setTime(h, m, s);
  }
}