﻿import util.*;
import mvcclock.*;
import mvc.*;
import mx.controls.Button;

/**
 * 建立一個可以控制ClockModel的使用者介面
 */
class mvcclock.ClockTools extends AbstractView {
  private var startBtn:Button;
  private var stopBtn:Button;
  private var resetBtn:Button;

  /**
   * 建構子
   */
  public function ClockTools (m:Observable, c:Controller, 
                              target:MovieClip, depth:Number,
                              x:Number, y:Number) {
    // 呼叫設置MVC關係的父建構子
    super(m, c);

    // 建立使用者介面
    makeTools(target, depth, x, y);
  }

  /**
   * 傳回這個外觀的預設控制邏輯
   */
  public function defaultController (model:Observable):Controller {
    return new ClockController(model);
  }

  /**
   * 建立放置「開始」、「停止」和「重置」按鈕的影片片段實體 
   * 並且建立這些按鈕。
   *
   * @param   target   	用來建立tools_mc片段的影片片段
   * @param   depth    	被建立的tools_mc所在的深度
   * @param   x  		tools_mc在target裡的水平座標
   * @param   y  		tools_mc在target裡的垂直座標
   */
  public function makeTools (target:MovieClip, depth:Number,
                             x:Number, y:Number):Void {
    // 建立一個影片片段容器
    var tools_mc:MovieClip = target.createEmptyMovieClip("tools", depth);
    tools_mc._x = x;
    tools_mc._y = y;
 
    // 在這個影片片段容器中，建立使用者介面按鈕。
    startBtn = tools_mc.createClassObject(Button, "start", 0);
    startBtn.label = "開始";
    startBtn.enabled = false;
    startBtn.addEventListener("click", getController(  ));

    stopBtn = tools_mc.createClassObject(Button, "stop", 1);
    stopBtn.label = "停止";
    stopBtn.enabled = false;
    stopBtn.move(startBtn.width + 5, startBtn.y);
    stopBtn.addEventListener("click", getController(  ));

    resetBtn = tools_mc.createClassObject(Button, "reset", 2);
    resetBtn.label = "重置";
    resetBtn.move(stopBtn.x + stopBtn.width + 5, startBtn.y);
    resetBtn.addEventListener("click", getController(  ));
  }

  /**
   * 更新使用者介面的狀態
   * 由ClockModel自動呼叫
   *
   * @param   o		廣播更新訊息的ClockModel物件
   * @param   infoObj	描述發生在ClockModel中的改變的
   * 				ClockUpdate實體
   */
  public function update (o:Observable, infoObj:Object):Void {
    // 把通用的infoObj物件轉換成ClockUpdate資料類型
    var info:ClockUpdate = ClockUpdate(infoObj);

    // 如果時鐘處於停止狀態的話，就啟用「開始」鈕； 
    // 如果時鐘處於執行狀態的話，就啟用「停止」鈕。
    if (info.isRunning) {
      stopBtn.enabled = true;
      startBtn.enabled = false;
    } else {
      stopBtn.enabled = false;
      startBtn.enabled = true;
    }
  }
}